@@ -1,167 +1,160 @@
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
+
 "use strict";
 
 module.exports = {
-  "plugins": [
-    "react"
-  ],
-  "globals": {
-    "exports": true,
-    "isWorker": true,
-    "isReplaying": true,
-    "loader": true,
-    "module": true,
-    "reportError": true,
-    "require": true,
+  plugins: ["react"],
+  globals: {
+    exports: true,
+    isWorker: true,
+    loader: true,
+    module: true,
+    reportError: true,
+    require: true,
   },
-  "overrides": [{
-    "files": [
-      "client/framework/**",
-    ],
-    "rules": {
-      "no-return-assign": "off",
-    }
-  }, {
-    "files": [
-      "client/scratchpad/scratchpad-manager.jsm",
-      "client/scratchpad/scratchpad.js",
-      "client/shared/*.jsm",
-    ],
-    "rules": {
-      "camelcase": "off",
-    }
-  }, {
-    "files": [
-      "client/framework/**",
-      "client/scratchpad/**",
-      "client/shared/*.jsm",
-      "client/shared/widgets/*.jsm",
-      "client/webide/**",
-    ],
-    "rules": {
-      "consistent-return": "off",
-    }
-  }, {
-    "files": [
-      "client/framework/**",
-      "client/scratchpad/**",
-      "client/webide/**",
-    ],
-    "rules": {
-      "max-nested-callbacks": "off",
-    }
-  }, {
-    "files": [
-      "client/framework/**",
-      "client/scratchpad/**",
-      "client/shared/*.jsm",
-      "client/shared/widgets/*.jsm",
-      "client/webide/**",
-    ],
-    "rules": {
-      "max-len": "off",
-    }
-  }, {
-    "files": [
-      "client/scratchpad/test/browser_scratchpad_inspect.js",
-      "client/scratchpad/test/browser_scratchpad_inspect_primitives.js",
-    ],
-    "rules": {
-      "no-labels": "off",
-    }
-  }, {
-    "files": [
-      "client/framework/**",
-      "client/scratchpad/**",
-      "client/shared/*.jsm",
-      "client/shared/widgets/*.jsm",
-      "client/webide/**",
-    ],
-    "rules": {
-      "mozilla/no-aArgs": "off",
-    }
-  }, {
-    "files": [
-      "client/framework/test/**",
-      "client/scratchpad/**",
-    ],
-    "rules": {
-      "mozilla/var-only-at-top-level": "off",
-    }
-  }, {
-    "files": [
-      "client/framework/**",
-      "client/scratchpad/**",
-      "client/shared/widgets/*.jsm",
-      "client/webide/**",
-    ],
-    "rules": {
-      "no-shadow": "off",
-    }
-  }, {
-    "files": [
-      "client/framework/**",
-      "client/scratchpad/**",
-      "client/webide/**",
-    ],
-    "rules": {
-      "strict": "off",
-    }
-  }, {
-    // For all head*.js files, turn off no-unused-vars at a global level
-    "files": [
-      "**/head*.js",
-    ],
-    "rules": {
-      "no-unused-vars": ["error", {"args": "none", "vars": "local"}],
-    }
-  }, {
-    // Cu, Cc etc... are not available in most devtools modules loaded by require.
-    "files": [
-      "**"
-    ],
-    "excludedFiles": [
-      // Enable the rule on JSM, test head files and some specific files.
-      "**/*.jsm",
-      "**/test/**/head.js",
-      "**/test/**/shared-head.js",
-      "client/debugger/test/mochitest/code_frame-script.js",
-      "client/inspector/animation-old/test/doc_frame_script.js",
-      "client/inspector/animation/test/doc_frame_script.js",
-      "client/inspector/rules/test/doc_frame_script.js",
-      "client/inspector/shared/test/doc_frame_script.js",
-      "client/jsonview/converter-observer.js",
-      "client/jsonview/test/doc_frame_script.js",
-      "client/responsive.html/browser/content.js",
-      "client/shared/browser-loader.js",
-      "server/actors/webconsole/content-process-forward.js",
-      "server/actors/worker/service-worker-process.js",
-      "server/startup/content-process.js",
-      "server/startup/frame.js",
-      "shared/base-loader.js",
-      "shared/worker/loader.js",
-      "startup/aboutdebugging-registration.js",
-      "startup/aboutdevtools/aboutdevtools-registration.js",
-      "startup/aboutdevtoolstoolbox-registration.js",
-      "startup/devtools-startup.js",
-    ],
-    "rules": {
-      "mozilla/no-define-cc-etc": "off",
-    }
-  }, ],
-  "rules": {
+  overrides: [
+    {
+      files: ["client/framework/**"],
+      rules: {
+        "no-return-assign": "off",
+      },
+    },
+    {
+      files: [
+        "client/shared/*.jsm",
+        // Allow non-camelcase so that run_test doesn't produce a warning.
+        "**/test*/**/*",
+      ],
+      rules: {
+        camelcase: "off",
+      },
+    },
+    {
+      files: [
+        "client/framework/**",
+        "client/shared/*.jsm",
+        "client/shared/widgets/*.jsm",
+        "client/storage/VariablesView.jsm",
+      ],
+      rules: {
+        "consistent-return": "off",
+      },
+    },
+    {
+      files: ["client/framework/**"],
+      rules: {
+        "max-nested-callbacks": "off",
+      },
+    },
+    {
+      files: [
+        "client/framework/**",
+        "client/shared/*.jsm",
+        "client/shared/widgets/*.jsm",
+        "client/storage/VariablesView.jsm",
+        "shared/webconsole/test/chrome/*.html",
+      ],
+      rules: {
+        "mozilla/no-aArgs": "off",
+      },
+    },
+    {
+      files: ["client/framework/test/**"],
+      rules: {
+        "mozilla/var-only-at-top-level": "off",
+      },
+    },
+    {
+      files: [
+        "client/framework/**",
+        "client/shared/widgets/*.jsm",
+        "client/storage/VariablesView.jsm",
+      ],
+      rules: {
+        "no-shadow": "off",
+      },
+    },
+    {
+      files: ["client/framework/**"],
+      rules: {
+        strict: "off",
+      },
+    },
+    {
+      // For all head*.js files, turn off no-unused-vars at a global level
+      files: ["**/head*.js"],
+      rules: {
+        "no-unused-vars": ["error", { args: "none", vars: "local" }],
+      },
+    },
+    {
+      // For all server and shared files, prevent requiring devtools/client
+      // modules.
+      files: ["server/**", "shared/**"],
+      rules: {
+        "mozilla/reject-some-requires": [
+          "error",
+          "^(resource://)?devtools/client",
+        ],
+      },
+      excludedFiles: [
+        // Tests can always import anything.
+        "**/test*/**/*",
+      ],
+    },
+    {
+      // Cu, Cc etc... are not available in most devtools modules loaded by require.
+      files: ["**"],
+      excludedFiles: [
+        // Enable the rule on JSM, test head files and some specific files.
+        "**/*.jsm",
+        "**/*.sjs",
+        "**/test/**/head.js",
+        "**/test/**/shared-head.js",
+        "client/debugger/test/mochitest/code_frame-script.js",
+        "client/responsive.html/browser/content.js",
+        "server/actors/webconsole/content-process-forward.js",
+        "server/startup/content-process.js",
+        "server/startup/frame.js",
+        "shared/loader/base-loader.js",
+        "shared/loader/browser-loader.js",
+        "shared/loader/worker-loader.js",
+        "startup/aboutdebugging-registration.js",
+        "startup/aboutdevtoolstoolbox-registration.js",
+        "startup/devtools-startup.js",
+      ],
+      rules: {
+        "mozilla/no-define-cc-etc": "off",
+      },
+    },
+    {
+      // All DevTools files should avoid relative paths.
+      files: ["**"],
+      excludedFiles: [
+        // Debugger modules have a custom bundling logic which relies on relative
+        // paths.
+        "client/debugger/src/**",
+        // `client/shared/build` contains node helpers to build the debugger and
+        // not devtools modules.
+        "client/shared/build/**",
+      ],
+      rules: {
+        "mozilla/reject-relative-requires": "error",
+      },
+    },
+  ],
+  rules: {
     // These are the rules that have been configured so far to match the
     // devtools coding style.
 
     // Rules from the mozilla plugin
+    "mozilla/balanced-observers": "error",
     "mozilla/no-aArgs": "error",
     // See bug 1224289.
     "mozilla/reject-importGlobalProperties": ["error", "everything"],
-    // devtools/shared/platform is special; see the README.md in that
-    // directory for details.  We reject requires using explicit
-    // subdirectories of this directory.
-    "mozilla/reject-some-requires": ["error", "^devtools/shared/platform/(chome|content)/"],
     "mozilla/var-only-at-top-level": "error",
-    "mozilla/use-chromeutils-import": ["error", {allowCu: true}],
 
     // Rules from the React plugin
     "react/display-name": "error",
@@ -172,63 +165,56 @@ module.exports = {
     "react/no-unknown-property": "error",
     "react/prefer-es6-class": ["off", "always"],
     "react/prop-types": "error",
-    "react/sort-comp": ["error", {
-      order: [
-        "static-methods",
-        "lifecycle",
-        "everything-else",
-        "render"
-      ],
-      groups: {
-        lifecycle: [
-          "displayName",
-          "propTypes",
-          "contextTypes",
-          "childContextTypes",
-          "mixins",
-          "statics",
-          "defaultProps",
-          "constructor",
-          "getDefaultProps",
-          "getInitialState",
-          "state",
-          "getChildContext",
-          "componentWillMount",
-          "componentDidMount",
-          "componentWillReceiveProps",
-          "shouldComponentUpdate",
-          "componentWillUpdate",
-          "componentDidUpdate",
-          "componentWillUnmount"
-        ]
-      }
-    }],
+    "react/sort-comp": [
+      "error",
+      {
+        order: ["static-methods", "lifecycle", "everything-else", "render"],
+        groups: {
+          lifecycle: [
+            "displayName",
+            "propTypes",
+            "contextTypes",
+            "childContextTypes",
+            "mixins",
+            "statics",
+            "defaultProps",
+            "constructor",
+            "getDefaultProps",
+            "getInitialState",
+            "state",
+            "getChildContext",
+            "componentWillMount",
+            "componentDidMount",
+            "componentWillReceiveProps",
+            "shouldComponentUpdate",
+            "componentWillUpdate",
+            "componentDidUpdate",
+            "componentWillUnmount",
+          ],
+        },
+      },
+    ],
 
     // Disallow using variables outside the blocks they are defined (especially
     // since only let and const are used, see "no-var").
     "block-scoped-var": "error",
-    // Enforce one true brace style (opening brace on the same line) and avoid
-    // start and end braces on the same line.
-    "brace-style": ["error", "1tbs", {"allowSingleLine": false}],
     // Require camel case names
-    "camelcase": "error",
+    camelcase: ["error", { properties: "never" }],
     // Warn about cyclomatic complexity in functions.
-    "complexity": ["error", 53],
+    // 20 is ESLint's default, and we want to keep it this way to prevent new highly
+    // complex functions from being introduced. However, because Mozilla's eslintrc has
+    // some other value defined, we need to override it here. See bug 1553449 for more
+    // information on complex DevTools functions that are currently excluded.
+    complexity: ["error", 20],
     // Don't warn for inconsistent naming when capturing this (not so important
     // with auto-binding fat arrow functions).
     "consistent-this": "off",
-    // Enforce curly brace conventions for all control statements.
-    "curly": "error",
     // Don't require a default case in switch statements. Avoid being forced to
     // add a bogus default when you know all possible cases are handled.
     "default-case": "off",
-    // Enforce dots on the next line with property name.
-    "dot-location": ["error", "property"],
     // Allow using == instead of ===, in the interest of landing something since
     // the devtools codebase is split on convention here.
-    "eqeqeq": "off",
-    // Disallow space between function identifier and application.
-    "func-call-spacing": "error",
+    eqeqeq: "off",
     // Don't require function expressions to have a name.
     // This makes the code more verbose and hard to read. Our engine already
     // does a fantastic job assigning a name to the function, which includes
@@ -239,15 +225,8 @@ module.exports = {
     "func-style": "off",
     // Only useful in a node environment.
     "handle-callback-err": "off",
-    // Tab width.
-    "indent-legacy": ["error", 2, {"SwitchCase": 1, "ArrayExpression": "first", "ObjectExpression": "first"}],
-    // Enforces spacing between keys and values in object literal properties.
-    "key-spacing": ["error", {"beforeColon": false, "afterColon": true}],
-    // Maximum length of a line.
-    "max-len": ["error", 90, 2, {
-      "ignoreUrls": true,
-      "ignorePattern": "data:image\/|\\s*require\\s*\\(|^\\s*loader\\.lazy|-\\*-"
-    }],
+    // componentDidUnmount is not a real lifecycle method, use componentWillUnmount.
+    "id-denylist": ["error", "componentDidUnmount"],
     // Maximum depth callbacks can be nested.
     "max-nested-callbacks": ["error", 3],
     // Don't limit the number of parameters that can be used in a function.
@@ -258,10 +237,7 @@ module.exports = {
     // Require a capital letter for constructors, only check if all new
     // operators are followed by a capital letter. Don't warn when capitalized
     // functions are used without the new operator.
-    "new-cap": ["error", {"capIsNew": false}],
-    // Disallow the omission of parentheses when invoking a constructor with no
-    // arguments.
-    "new-parens": "error",
+    "new-cap": ["error", { capIsNew: false }],
     // Allow use of bitwise operators.
     "no-bitwise": "off",
     // Allow using the console API.
@@ -280,24 +256,14 @@ module.exports = {
     "no-empty": "error",
     // Disallow adding to native types
     "no-extend-native": "error",
-    // Allow unnecessary parentheses, as they may make the code more readable.
-    "no-extra-parens": "off",
     // Disallow fallthrough of case statements, except if there is a comment.
     "no-fallthrough": "error",
-    // Allow the use of leading or trailing decimal points in numeric literals.
-    "no-floating-decimal": "off",
     // Allow comments inline after code.
     "no-inline-comments": "off",
     // Allow mixing regular variable and require declarations (not a node env).
     "no-mixed-requires": "off",
-    // Disallow use of multiple spaces (sometimes used to align const values,
-    // array or object items, etc.). It's hard to maintain and doesn't add that
-    // much benefit.
-    "no-multi-spaces": "error",
     // Disallow use of multiline strings (use template strings instead).
     "no-multi-str": "error",
-    // Disallow multiple empty lines.
-    "no-multiple-empty-lines": ["error", {"max": 1}],
     // Allow use of new operator with the require function.
     "no-new-require": "off",
     // Allow reassignment of function parameters.
@@ -316,6 +282,14 @@ module.exports = {
     "no-regex-spaces": "off",
     // Don't restrict usage of specified node modules (not a node environment).
     "no-restricted-modules": "off",
+    // Prevent using some properties
+    "no-restricted-properties": [
+      "error",
+      {
+        property: "setupInParent",
+        message: "avoid child/parent communication with setupInParent",
+      },
+    ],
     // Disallow use of assignment in return statement. It is preferable for a
     // single line of code to have only one easily predictable effect.
     "no-return-assign": "error",
@@ -337,7 +311,7 @@ module.exports = {
     "no-undefined": "off",
     // Disallow global and local variables that aren't used, but allow unused
     // function arguments.
-    "no-unused-vars": ["error", {"args": "none", "vars": "all"}],
+    "no-unused-vars": ["error", { args: "none", vars: "all" }],
     // Allow using variables before they are defined.
     "no-use-before-define": "off",
     // We use var-only-at-top-level instead of no-var as we allow top level
@@ -352,23 +326,14 @@ module.exports = {
     // Allow more than one variable declaration per function.
     "one-var": "off",
     // Enforce using `let` only when variables are reassigned.
-    "prefer-const": ["error", { "destructuring": "all" }],
-    // Don't require quotes around object literal property names.
-    "quote-props": "off",
+    "prefer-const": ["error", { destructuring: "all" }],
     // Require use of the second argument for parseInt().
-    "radix": "error",
-    // Enforce spacing after semicolons.
-    "semi-spacing": ["error", {"before": false, "after": true}],
+    radix: "error",
     // Don't require to sort variables within the same declaration block.
     // Anyway, one-var is disabled.
     "sort-vars": "off",
-    // Disallow spaces inside parentheses.
-    "space-in-parens": ["error", "never"],
-    // Require spaces before/after unary operators (words on by default,
-    // nonwords off by default).
-    "space-unary-ops": ["error", { "words": true, "nonwords": false }],
     // Require "use strict" to be defined globally in the script.
-    "strict": ["error", "global"],
+    strict: ["error", "global"],
     // Warn about invalid JSDoc comments.
     // Disabled for now because of https://github.com/eslint/eslint/issues/2270
     // The rule fails on some jsdoc comments like in:
@@ -376,21 +341,12 @@ module.exports = {
     "valid-jsdoc": "off",
     // Allow vars to be declared anywhere in the scope.
     "vars-on-top": "off",
-    // Don't require immediate function invocation to be wrapped in parentheses.
-    "wrap-iife": "off",
-    // Don't require regex literals to be wrapped in parentheses (which
-    // supposedly prevent them from being mistaken for division operators).
-    "wrap-regex": "off",
     // Disallow Yoda conditions (where literal value comes first).
-    "yoda": "error",
+    yoda: "error",
 
     // And these are the rules that haven't been discussed so far, and that are
     // disabled for now until we introduce them, one at a time.
 
-    // enforce consistent spacing before and after the arrow in arrow functions
-    "arrow-spacing": "off",
-    // enforce consistent spacing inside computed property brackets
-    "computed-property-spacing": "off",
     // Require for-in loops to have an if statement.
     "guard-for-in": "off",
     // disallow the use of alert, confirm, and prompt
@@ -405,8 +361,6 @@ module.exports = {
     "no-invalid-regexp": "off",
     // disallow irregular whitespace outside of strings and comments
     "no-irregular-whitespace": "off",
-    // disallow usage of __iterator__ property
-    "no-iterator": "off",
     // disallow labels that share a name with a variable
     "no-label-var": "off",
     // disallow unnecessary nested blocks
@@ -439,7 +393,10 @@ module.exports = {
     // require assignment operator shorthand where possible or prohibit it
     // entirely
     "operator-assignment": "off",
-    // enforce operators to be placed before or after line breaks
-    "operator-linebreak": "off",
-  }
+  },
+  settings: {
+    react: {
+      version: "16.8",
+    },
+  },
 };
