@@ -3,76 +3,64 @@
 
 /* import-globals-from ../../../shared/test/shared-head.js */
 /* import-globals-from ../../../inspector/test/shared-head.js */
+/* import-globals-from ../../../shared/test/telemetry-test-helpers.js */
 
 /* global waitUntilState, gBrowser */
 /* exported addTestTab, checkTreeState, checkSidebarState, checkAuditState, selectRow,
-            toggleRow, toggleFilter, addA11yPanelTestsTask, reload,
-            navigate */
+            toggleRow, toggleMenuItem, addA11yPanelTestsTask, navigate,
+            openSimulationMenu, toggleSimulationOption, TREE_FILTERS_MENU_ID,
+            PREFS_MENU_ID */
 
 "use strict";
 
 // Import framework's shared head.
 Services.scriptloader.loadSubScript(
   "chrome://mochitests/content/browser/devtools/client/shared/test/shared-head.js",
-  this);
+  this
+);
 
 // Import inspector's shared head.
 Services.scriptloader.loadSubScript(
   "chrome://mochitests/content/browser/devtools/client/inspector/test/shared-head.js",
-  this);
+  this
+);
 
-// Load the shared Redux helpers into this compartment.
-Services.scriptloader.loadSubScript(
-  "chrome://mochitests/content/browser/devtools/client/shared/test/shared-redux-head.js",
-  this);
-
-const { ORDERED_PROPS } = require("devtools/client/accessibility/constants");
+const {
+  ORDERED_PROPS,
+  PREF_KEYS,
+} = require("devtools/client/accessibility/constants");
 
 // Enable the Accessibility panel
 Services.prefs.setBoolPref("devtools.accessibility.enabled", true);
 
-/**
- * Enable accessibility service and wait for a11y init event.
- * @return {Object}  instance of accessibility service.
- */
-async function initA11y() {
-  if (Services.appinfo.accessibilityEnabled) {
-    return Cc["@mozilla.org/accessibilityService;1"].getService(
-      Ci.nsIAccessibilityService);
-  }
+const SIMULATION_MENU_BUTTON_ID = "#simulation-menu-button";
+const TREE_FILTERS_MENU_ID = "accessibility-tree-filters-menu";
+const PREFS_MENU_ID = "accessibility-tree-filters-prefs-menu";
 
-  const initPromise = new Promise(resolve => {
-    const observe = () => {
-      Services.obs.removeObserver(observe, "a11y-init-or-shutdown");
-      resolve();
-    };
-    Services.obs.addObserver(observe, "a11y-init-or-shutdown");
-  });
-
-  const a11yService = Cc["@mozilla.org/accessibilityService;1"].getService(
-    Ci.nsIAccessibilityService);
-  await initPromise;
-  return a11yService;
-}
+const MENU_INDEXES = {
+  [TREE_FILTERS_MENU_ID]: 0,
+  [PREFS_MENU_ID]: 1,
+};
 
 /**
  * Wait for accessibility service to shut down. We consider it shut down when
  * an "a11y-init-or-shutdown" event is received with a value of "0".
  */
-function shutdownA11y() {
-  if (!Services.appinfo.accessibilityEnabled) {
-    return Promise.resolve();
-  }
-
-  // Force collections to speed up accessibility service shutdown.
-  Cu.forceGC();
-  Cu.forceCC();
-  Cu.forceShrinkingGC();
-
+function waitForAccessibilityShutdown() {
   return new Promise(resolve => {
+    if (!Services.appinfo.accessibilityEnabled) {
+      resolve();
+      return;
+    }
+
     const observe = (subject, topic, data) => {
       if (data === "0") {
         Services.obs.removeObserver(observe, "a11y-init-or-shutdown");
+        // Sanity check
+        ok(
+          !Services.appinfo.accessibilityEnabled,
+          "Accessibility disabled in this process"
+        );
         resolve();
       }
     };
@@ -81,12 +69,24 @@ function shutdownA11y() {
     // accessibility service naturally if there are no more XPCOM references to
     // a11y related objects (after GC/CC).
     Services.obs.addObserver(observe, "a11y-init-or-shutdown");
+
+    // Force garbage collection.
+    SpecialPowers.gc();
+    SpecialPowers.forceShrinkingGC();
+    SpecialPowers.forceCC();
   });
 }
 
+/**
+ * Ensure that accessibility is completely shutdown.
+ */
+async function shutdownAccessibility(browser) {
+  await waitForAccessibilityShutdown();
+  await SpecialPowers.spawn(browser, [], waitForAccessibilityShutdown);
+}
+
 registerCleanupFunction(async () => {
   info("Cleaning up...");
-  await shutdownA11y();
   Services.prefs.clearUserPref("devtools.accessibility.enabled");
 });
 
@@ -114,13 +114,13 @@ async function addTestTab(url) {
     EventUtils.sendMouseEvent({ type: "click" }, enableButton, win);
   }
 
-  await waitUntilState(store, state =>
-    state.accessibles.size === 1 && state.details.accessible &&
-    state.details.accessible.role === "document");
-
-  // Wait for inspector load here to avoid protocol errors on shutdown, since
-  // accessibility panel test can be too fast.
-  await win.gToolbox.loadTool("inspector");
+  await waitUntilState(
+    store,
+    state =>
+      state.accessibles.size === 1 &&
+      state.details.accessible &&
+      state.details.accessible.role === "document"
+  );
 
   return {
     tab,
@@ -133,21 +133,6 @@ async function addTestTab(url) {
   };
 }
 
-/**
- * Turn off accessibility features from within the panel. We call it before the
- * cleanup function to make sure that the panel is still present.
- */
-async function disableAccessibilityInspector(env) {
-  const { doc, win, panel } = env;
-  // Disable accessibility service through the panel and wait for the shutdown
-  // event.
-  const shutdown = panel.front.once("shutdown");
-  const disableButton = await BrowserTestUtils.waitForCondition(() =>
-    doc.getElementById("accessibility-disable-button"), "Wait for the disable button.");
-  EventUtils.sendMouseEvent({ type: "click" }, disableButton, win);
-  await shutdown;
-}
-
 /**
  * Open the Accessibility panel for the given tab.
  *
@@ -157,8 +142,9 @@ async function disableAccessibilityInspector(env) {
  * @return a promise that is resolved once the panel is open.
  */
 async function initAccessibilityPanel(tab = gBrowser.selectedTab) {
-  const target = await TargetFactory.forTab(tab);
-  const toolbox = await gDevTools.showToolbox(target, "accessibility");
+  const toolbox = await gDevTools.showToolboxForTab(tab, {
+    toolId: "accessibility",
+  });
   return toolbox.getCurrentPanel();
 }
 
@@ -172,8 +158,10 @@ async function initAccessibilityPanel(tab = gBrowser.selectedTab) {
  */
 function compareBadges(badges, expected = []) {
   const badgeEls = badges ? [...badges.querySelectorAll(".badge")] : [];
-  return badgeEls.length === expected.length &&
-         badgeEls.every((badge, i) => badge.textContent === expected[i]);
+  return (
+    badgeEls.length === expected.length &&
+    badgeEls.every((badge, i) => badge.textContent === expected[i])
+  );
 }
 
 /**
@@ -207,10 +195,13 @@ function closestScrolledParent(node) {
 function isVisible(element) {
   const { top, bottom } = element.getBoundingClientRect();
   const scrolledParent = closestScrolledParent(element.parentNode);
-  const scrolledParentRect = scrolledParent ? scrolledParent.getBoundingClientRect() :
-                                              null;
-  return !scrolledParent ||
-         (top >= scrolledParentRect.top && bottom <= scrolledParentRect.bottom);
+  const scrolledParentRect = scrolledParent
+    ? scrolledParent.getBoundingClientRect()
+    : null;
+  return (
+    !scrolledParent ||
+    (top >= scrolledParentRect.top && bottom <= scrolledParentRect.bottom)
+  );
 }
 
 /**
@@ -261,15 +252,23 @@ function checkLevel(row, expected) {
  */
 async function checkTreeState(doc, expected) {
   info("Checking tree state.");
-  const hasExpectedStructure = await BrowserTestUtils.waitForCondition(() =>
-    [...doc.querySelectorAll(".treeRow")].every((row, i) => {
+  const hasExpectedStructure = await BrowserTestUtils.waitForCondition(() => {
+    const rows = [...doc.querySelectorAll(".treeRow")];
+    if (rows.length !== expected.length) {
+      return false;
+    }
+
+    return rows.every((row, i) => {
       const { role, name, badges, selected, level } = expected[i];
-      return row.querySelector(".treeLabelCell").textContent === role &&
+      return (
+        row.querySelector(".treeLabelCell").textContent === role &&
         row.querySelector(".treeValueCell").textContent === name &&
         compareBadges(row.querySelector(".badges"), badges) &&
         checkSelected(row, selected) &&
-        checkLevel(row, level);
-    }), "Wait for the right tree update.");
+        checkLevel(row, level)
+      );
+    });
+  }, "Wait for the right tree update.");
 
   ok(hasExpectedStructure, "Tree structure is correct.");
 }
@@ -287,12 +286,17 @@ function relationsMatch(relations, expected) {
     let expTargets = expected[relationType];
     expTargets = Array.isArray(expTargets) ? expTargets : [expTargets];
 
-    let targets = relations[relationType];
+    let targets = relations ? relations[relationType] : [];
     targets = Array.isArray(targets) ? targets : [targets];
 
     for (const index in expTargets) {
-      if (expTargets[index].name !== targets[index].name ||
-          expTargets[index].role !== targets[index].role) {
+      if (!targets[index]) {
+        return false;
+      }
+      if (
+        expTargets[index].name !== targets[index].name ||
+        expTargets[index].role !== targets[index].role
+      ) {
         return false;
       }
     }
@@ -333,8 +337,10 @@ async function checkAuditState(store, expectedState) {
     for (const key in expectedState) {
       const expected = expectedState[key];
       if (expected && typeof expected === "object") {
-        if (JSON.stringify(audit[key], parseNumReplacer) !==
-            JSON.stringify(expected, parseNumReplacer)) {
+        if (
+          JSON.stringify(audit[key], parseNumReplacer) !==
+          JSON.stringify(expected, parseNumReplacer)
+        ) {
           return false;
         }
       } else if (audit && audit[key] !== expected) {
@@ -367,7 +373,9 @@ async function checkSidebarState(store, expectedState) {
           return false;
         }
       } else if (EXPANDABLE_PROPS.includes(key)) {
-        if (JSON.stringify(details.accessible[key]) !== JSON.stringify(expected)) {
+        if (
+          JSON.stringify(details.accessible[key]) !== JSON.stringify(expected)
+        ) {
           return false;
         }
       } else if (details.accessible && details.accessible[key] !== expected) {
@@ -380,6 +388,103 @@ async function checkSidebarState(store, expectedState) {
   });
 }
 
+/**
+ * Check the state of the accessibility related prefs.
+ * @param  {Document} doc
+ *         accessibility inspector panel document.
+ * @param  {Object}   toolbarPrefValues
+ *         Expected state of the panel prefs as well as the redux state that
+ *         keeps track of it. Includes:
+ *         - SCROLL_INTO_VIEW (devtools.accessibility.scroll-into-view)
+ * @param  {Object}   store
+ *         React store for the panel (includes store for the sidebar).
+ */
+async function checkToolbarPrefsState(doc, toolbarPrefValues, store) {
+  info("Checking toolbar prefs state.");
+  const [hasExpectedStructure] = await Promise.all([
+    // Check that appropriate preferences are set as expected.
+    BrowserTestUtils.waitForCondition(() => {
+      return Object.keys(toolbarPrefValues).every(
+        name =>
+          Services.prefs.getBoolPref(PREF_KEYS[name], false) ===
+          toolbarPrefValues[name]
+      );
+    }, "Wait for the right prefs state."),
+    // Check that ui state is set as expected.
+    waitUntilState(store, ({ ui }) => {
+      for (const name in toolbarPrefValues) {
+        if (ui[name] !== toolbarPrefValues[name]) {
+          return false;
+        }
+      }
+
+      ok(true, "UI pref state is correct.");
+      return true;
+    }),
+  ]);
+  ok(hasExpectedStructure, "Prefs state is correct.");
+}
+
+/**
+ * Check the state of the accessibility checks toolbar.
+ * @param  {Object} store
+ *         React store for the panel (includes store for the sidebar).
+ * @param  {Object} activeToolbarFilters
+ *         Expected active state of the filters in the toolbar.
+ */
+async function checkToolbarState(doc, activeToolbarFilters) {
+  info("Checking toolbar state.");
+  const hasExpectedStructure = await BrowserTestUtils.waitForCondition(
+    () =>
+      [
+        ...doc.querySelectorAll("#accessibility-tree-filters-menu .command"),
+      ].every(
+        (filter, i) =>
+          (activeToolbarFilters[i] ? "true" : null) ===
+          filter.getAttribute("aria-checked")
+      ),
+    "Wait for the right toolbar state."
+  );
+
+  ok(hasExpectedStructure, "Toolbar state is correct.");
+}
+
+/**
+ * Check the state of the simulation button and menu components.
+ * @param  {Object} doc         Panel document.
+ * @param  {Object} expected    Expected states of the simulation components:
+ *                              menuVisible, buttonActive, checkedOptionIndices (Optional)
+ */
+async function checkSimulationState(doc, expected) {
+  const { buttonActive, checkedOptionIndices } = expected;
+  const simulationMenuOptions = doc
+    .querySelector(SIMULATION_MENU_BUTTON_ID + "-menu")
+    .querySelectorAll(".menuitem");
+
+  // Check simulation menu button state
+  is(
+    doc.querySelector(SIMULATION_MENU_BUTTON_ID).className,
+    `devtools-button toolbar-menu-button simulation${
+      buttonActive ? " active" : ""
+    }`,
+    `Simulation menu button contains ${buttonActive ? "active" : "base"} class.`
+  );
+
+  // Check simulation menu options states, if specified
+  if (checkedOptionIndices) {
+    simulationMenuOptions.forEach((menuListItem, index) => {
+      const isChecked = checkedOptionIndices.includes(index);
+      const button = menuListItem.firstChild;
+
+      is(
+        button.getAttribute("aria-checked"),
+        isChecked ? "true" : null,
+        `Simulation option ${index} is ${isChecked ? "" : "not "}selected.`
+      );
+    });
+  }
+}
+
 /**
  * Focus accessibility properties tree in the a11y inspector sidebar. If focused for the
  * first time, the tree will select first rendered node as defult selection for keyboard
@@ -391,8 +496,10 @@ async function focusAccessibleProperties(doc) {
   const tree = doc.querySelector(".tree");
   if (doc.activeElement !== tree) {
     tree.focus();
-    await BrowserTestUtils.waitForCondition(() =>
-      tree.querySelector(".node.focused"), "Tree selected.");
+    await BrowserTestUtils.waitForCondition(
+      () => tree.querySelector(".node.focused"),
+      "Tree selected."
+    );
   }
 }
 
@@ -415,7 +522,13 @@ async function selectProperty(doc, id) {
         return node.firstChild.classList.contains("focused");
       }
 
+      AccessibilityUtils.setEnv({
+        // Keyboard navigation is handled on the container level using arrow
+        // keys.
+        nonNegativeTabIndexRule: false,
+      });
       EventUtils.sendMouseEvent({ type: "click" }, node, win);
+      AccessibilityUtils.resetEnv();
       selected = true;
     } else {
       const tree = doc.querySelector(".tree");
@@ -435,8 +548,16 @@ async function selectProperty(doc, id) {
  */
 function selectRow(doc, rowNumber) {
   info(`Selecting row ${rowNumber}.`);
-  EventUtils.sendMouseEvent({ type: "click" },
-    doc.querySelectorAll(".treeRow")[rowNumber], doc.defaultView);
+  AccessibilityUtils.setEnv({
+    // Keyboard navigation is handled on the container level using arrow keys.
+    nonNegativeTabIndexRule: false,
+  });
+  EventUtils.sendMouseEvent(
+    { type: "click" },
+    doc.querySelectorAll(".treeRow")[rowNumber],
+    doc.defaultView
+  );
+  AccessibilityUtils.resetEnv();
 }
 
 /**
@@ -452,34 +573,100 @@ async function toggleRow(doc, rowNumber) {
 
   info(`${expected ? "Expanding" : "Collapsing"} row ${rowNumber}.`);
 
+  AccessibilityUtils.setEnv({
+    // We intentionally remove the twisty from the accessibility tree in the
+    // TreeView component and handle keyboard navigation using the arrow keys.
+    mustHaveAccessibleRule: false,
+  });
   EventUtils.sendMouseEvent({ type: "click" }, twisty, win);
-  await BrowserTestUtils.waitForCondition(() =>
-    !twisty.classList.contains("devtools-throbber") &&
-    expected === twisty.classList.contains("open"), "Twisty updated.");
+  AccessibilityUtils.resetEnv();
+  await BrowserTestUtils.waitForCondition(
+    () =>
+      !twisty.classList.contains("devtools-throbber") &&
+      expected === twisty.classList.contains("open"),
+    "Twisty updated."
+  );
 }
 
 /**
- * Toggle an accessibility audit filter based on its index in the toolbar.
- * @param  {document} doc         panel documnent.
- * @param  {Number}   filterIndex index of the filter to be toggled.
+ * Toggle a specific menu item based on its index in the menu.
+ * @param  {document} toolboxDoc
+ *         toolbox document.
+ * @param  {document} doc
+ *         panel document.
+ * @param  {String} menuId
+ *         The id of the menu (menuId passed to the MenuButton component)
+ * @param  {Number}   menuItemIndex
+ *         index of the menu item to be toggled.
  */
-async function toggleFilter(doc, filterIndex) {
-  const win = doc.defaultView;
-  const filter = doc.querySelectorAll(
-    ".devtools-toolbar .badge.toggle-button")[filterIndex];
-  const expected = !filter.classList.contains("checked");
+async function toggleMenuItem(doc, toolboxDoc, menuId, menuItemIndex) {
+  const toolboxWin = toolboxDoc.defaultView;
+  const panelWin = doc.defaultView;
 
-  EventUtils.synthesizeMouseAtCenter(filter, {}, win);
-  await BrowserTestUtils.waitForCondition(() =>
-    expected === filter.classList.contains("checked"), "Filter updated.");
+  const menuButton = doc.querySelectorAll(".toolbar-menu-button")[
+    MENU_INDEXES[menuId]
+  ];
+  ok(menuButton, "Expected menu button");
+
+  const menuEl = toolboxDoc.getElementById(menuId);
+  const menuItem = menuEl.querySelectorAll(".command")[menuItemIndex];
+  ok(menuItem, "Expected menu item");
+
+  const expected =
+    menuItem.getAttribute("aria-checked") === "true" ? null : "true";
+
+  // Make the menu visible first.
+  const onPopupShown = new Promise(r =>
+    toolboxDoc.addEventListener("popupshown", r, { once: true })
+  );
+  EventUtils.synthesizeMouseAtCenter(menuButton, {}, panelWin);
+  await onPopupShown;
+  const boundingRect = menuItem.getBoundingClientRect();
+  ok(
+    boundingRect.width > 0 && boundingRect.height > 0,
+    "Menu item is visible."
+  );
+
+  EventUtils.synthesizeMouseAtCenter(menuItem, {}, toolboxWin);
+  await BrowserTestUtils.waitForCondition(
+    () => expected === menuItem.getAttribute("aria-checked"),
+    "Menu item updated."
+  );
 }
 
-async function findAccessibleFor({
-  toolbox: { walker: domWalker },
-  panel: { walker: a11yWalker },
-}, selector) {
+async function openSimulationMenu(doc) {
+  doc.querySelector(SIMULATION_MENU_BUTTON_ID).click();
+
+  await BrowserTestUtils.waitForCondition(() =>
+    doc
+      .querySelector(SIMULATION_MENU_BUTTON_ID + "-menu")
+      .classList.contains("tooltip-visible")
+  );
+}
+
+async function toggleSimulationOption(doc, optionIndex) {
+  const simulationMenu = doc.querySelector(SIMULATION_MENU_BUTTON_ID + "-menu");
+  simulationMenu.querySelectorAll(".menuitem")[optionIndex].firstChild.click();
+
+  await BrowserTestUtils.waitForCondition(
+    () => !simulationMenu.classList.contains("tooltip-visible")
+  );
+}
+
+async function findAccessibleFor(
+  {
+    toolbox: { target },
+    panel: {
+      accessibilityProxy: {
+        accessibilityFront: { accessibleWalkerFront },
+      },
+    },
+  },
+  selector
+) {
+  const domWalker = (await target.getFront("inspector")).walker;
   const node = await domWalker.querySelector(domWalker.rootNode, selector);
-  return a11yWalker.getAccessibleFor(node);
+  return accessibleWalkerFront.getAccessibleFor(node);
 }
 
 async function selectAccessibleForNode(env, selector) {
@@ -494,17 +681,28 @@ async function selectAccessibleForNode(env, selector) {
 /**
  * Iterate over setups/tests structure and test the state of the
  * accessibility panel.
- * @param  {JSON}   tests test data that has the format of:
- *                    {
- *                      desc     {String}    description for better logging
- *                      setup    {Function}  An optional setup that needs to be
- *                                           performed before the state of the
- *                                           tree and the sidebar can be checked
- *                      expected {JSON}      An expected states for the tree and
- *                                           the sidebar
- *                    }
- * @param  {Object} env  contains all relevant environment objects (same
- *                       structure as the return value of 'addTestTab' funciton)
+ * @param  {JSON}   tests
+ *         test data that has the format of:
+ *         {
+ *           desc     {String}    description for better logging
+ *           setup    {Function}  An optional setup that needs to be
+ *                                performed before the state of the
+ *                                tree and the sidebar can be checked
+ *           expected {JSON}      An expected states for parts of
+ *                                accessibility panel:
+ *            - tree:                 state of the accessibility tree widget
+ *            - sidebar:              state of the accessibility panel sidebar
+ *            - audit:                state of the audit redux state of the
+ *                                    panel
+ *            - toolbarPrefValues:    state of the accessibility panel
+ *                                    toolbar prefs and corresponding user
+ *                                    preferences.
+ *            - activeToolbarFilters: state of the accessibility panel
+ *                                    toolbar filters.
+ *         }
+ * @param  {Object} env
+ *         contains all relevant environment objects (same structure as the
+ *         return value of 'addTestTab' funciton)
  */
 async function runA11yPanelTests(tests, env) {
   for (const { desc, setup, expected } of tests) {
@@ -514,7 +712,14 @@ async function runA11yPanelTests(tests, env) {
       await setup(env);
     }
 
-    const { tree, sidebar, audit } = expected;
+    const {
+      tree,
+      sidebar,
+      audit,
+      toolbarPrefValues,
+      activeToolbarFilters,
+      simulation,
+    } = expected;
     if (tree) {
       await checkTreeState(env.doc, tree);
     }
@@ -523,24 +728,52 @@ async function runA11yPanelTests(tests, env) {
       await checkSidebarState(env.store, sidebar);
     }
 
+    if (activeToolbarFilters) {
+      await checkToolbarState(env.doc, activeToolbarFilters);
+    }
+
+    if (toolbarPrefValues) {
+      await checkToolbarPrefsState(env.doc, toolbarPrefValues, env.store);
+    }
+
     if (typeof audit !== "undefined") {
       await checkAuditState(env.store, audit);
     }
+
+    if (simulation) {
+      await checkSimulationState(env.doc, simulation);
+    }
   }
 }
 
 /**
  * Build a valid URL from an HTML snippet.
- * @param  {String} uri HTML snippet
+ * @param  {String}  uri      HTML snippet
+ * @param  {Object}  options  options for the test
  * @return {String}     built URL
  */
-function buildURL(uri) {
+function buildURL(uri, options = {}) {
+  if (options.remoteIframe) {
+    const srcURL = new URL(`http://example.net/document-builder.sjs`);
+    srcURL.searchParams.append(
+      "html",
+      `<html>
+        <head>
+          <meta charset="utf-8"/>
+          <title>Accessibility Panel Test (OOP)</title>
+        </head>
+        <body>${uri}</body>
+      </html>`
+    );
+    uri = `<iframe title="Accessibility Panel Test (OOP)" src="${srcURL.href}"/>`;
+  }
+
   return `data:text/html;charset=UTF-8,${encodeURIComponent(uri)}`;
 }
 
 /**
  * Add a test task based on the test structure and a test URL.
- * @param  {JSON}   tests  test data that has the format of:
+ * @param  {JSON}   tests    test data that has the format of:
  *                    {
  *                      desc     {String}    description for better logging
  *                      setup   {Function}   An optional setup that needs to be
@@ -549,11 +782,30 @@ function buildURL(uri) {
  *                      expected {JSON}      An expected states for the tree and
  *                                           the sidebar
  *                    }
- * @param {String}  uri    test URL
- * @param {String}  msg    a message that is printed for the test
+ * @param {String}  uri      test URL
+ * @param {String}  msg      a message that is printed for the test
+ * @param {Object}  options  options for the test
  */
-function addA11yPanelTestsTask(tests, uri, msg) {
-  addA11YPanelTask(msg, uri, env => runA11yPanelTests(tests, env));
+function addA11yPanelTestsTask(tests, uri, msg, options) {
+  addA11YPanelTask(msg, uri, env => runA11yPanelTests(tests, env), options);
+}
+
+/**
+ * Borrowed from framework's shared head. Close toolbox, completely disable
+ * accessibility and remove the tab.
+ * @param  {Tab}
+ *         tab The tab to close.
+ * @return {Promise}
+ *         Resolves when the toolbox and tab have been destroyed and closed.
+ */
+async function closeTabToolboxAccessibility(tab = gBrowser.selectedTab) {
+  if (TabDescriptorFactory.isKnownTab(tab)) {
+    await gDevTools.closeToolboxForTab(tab);
+  }
+
+  await shutdownAccessibility(gBrowser.getBrowserForTab(tab));
+  await removeTab(tab);
+  await new Promise(resolve => setTimeout(resolve, 0));
 }
 
 /**
@@ -562,33 +814,14 @@ function addA11yPanelTestsTask(tests, uri, msg) {
  * @param {String}   msg    a message that is printed for the test
  * @param {String}   uri    test URL
  * @param {Function} task   task function containing the tests.
+ * @param {Object}   options  options for the test
  */
-function addA11YPanelTask(msg, uri, task) {
+function addA11YPanelTask(msg, uri, task, options = {}) {
   add_task(async function a11YPanelTask() {
     info(msg);
-    const env = await addTestTab(buildURL(uri));
+
+    const env = await addTestTab(buildURL(uri, options));
     await task(env);
-    await disableAccessibilityInspector(env);
+    await closeTabToolboxAccessibility(env.tab);
   });
 }
-
-/**
- * Reload panel target.
- * @param  {Object} target             Panel target.
- * @param  {String} waitForTargetEvent Event to wait for after reload.
- */
-function reload(target, waitForTargetEvent = "navigate") {
-  executeSoon(() => target.reload());
-  return once(target, waitForTargetEvent);
-}
-
-/**
- * Navigate to a new URL within the panel target.
- * @param  {Object} target             Panel target.
- * @param  {Srting} url                URL to navigate to.
- * @param  {String} waitForTargetEvent Event to wait for after reload.
- */
-function navigate(target, url, waitForTargetEvent = "navigate") {
-  executeSoon(() => target.navigateTo({ url }));
-  return once(target, waitForTargetEvent);
-}
