@@ -7,13 +7,15 @@
 
 // There are shutdown issues for which multiple rejections are left uncaught.
 // See bug 1018184 for resolving these issues.
-const { PromiseTestUtils } = ChromeUtils.import("resource://testing-common/PromiseTestUtils.jsm");
-PromiseTestUtils.whitelistRejectionsGlobally(/this\.worker is null/);
+const { PromiseTestUtils } = ChromeUtils.import(
+  "resource://testing-common/PromiseTestUtils.jsm"
+);
+PromiseTestUtils.allowMatchingRejectionsGlobally(/this\.worker is null/);
 
 // Empty page
-const PAGE_URL = `${URL_ROOT}doc_empty-tab-01.html`;
-const JS_URL = `${URL_ROOT}code_binary_search_absolute.js`;
-const ORIGINAL_URL = `${URL_ROOT}code_binary_search.coffee`;
+const PAGE_URL = `${URL_ROOT_SSL}doc_empty-tab-01.html`;
+const JS_URL = `${URL_ROOT_SSL}code_binary_search_absolute.js`;
+const ORIGINAL_URL = `${URL_ROOT_SSL}code_binary_search.coffee`;
 
 add_task(async function() {
   const toolbox = await openNewTabAndToolbox(PAGE_URL, "jsdebugger");
@@ -25,8 +27,10 @@ add_task(async function() {
   await sourceSeen;
 
   info(`checking original location for ${JS_URL}:6`);
-  const newLoc = await service.originalPositionFor(JS_URL, 6, 4);
+  const newLoc = await new Promise(r =>
+    service.subscribeByURL(JS_URL, 6, 4, r)
+  );
 
-  is(newLoc.sourceUrl, ORIGINAL_URL, "check mapped URL");
+  is(newLoc.url, ORIGINAL_URL, "check mapped URL");
   is(newLoc.line, 4, "check mapped line number");
 });
