@@ -5,8 +5,8 @@
 
 "use strict";
 
-const PAGE_URL = `${URL_ROOT}doc_empty-tab-01.html`;
-const JS_URL = URL_ROOT + "code_bundle_late_script.js";
+const PAGE_URL = `${URL_ROOT_SSL}doc_empty-tab-01.html`;
+const JS_URL = URL_ROOT_SSL + "code_bundle_late_script.js";
 
 const ORIGINAL_URL = "webpack:///code_late_script.js";
 
@@ -21,15 +21,17 @@ add_task(async function() {
 
   const scriptMapped = new Promise(resolve => {
     let count = 0;
-    service.subscribe(
+    service.subscribeByURL(
       JS_URL,
       GENERATED_LINE,
       undefined,
-      (...args) => {
+      originalLocation => {
         if (count === 0) {
-          resolve(args);
+          resolve(originalLocation);
         }
         count += 1;
+
+        return () => {};
       }
     );
   });
@@ -40,8 +42,7 @@ add_task(async function() {
   await sourceSeen;
 
   // Ensure that the URL service fired an event about the location loading.
-  const [isSourceMapped, url, line] = await scriptMapped;
-  is(isSourceMapped, true, "check is mapped");
+  const { url, line } = await scriptMapped;
   is(url, ORIGINAL_URL, "check mapped URL");
   is(line, ORIGINAL_LINE, "check mapped line number");
 
