@@ -1,4 +1,3 @@
-/* vim: set ft=javascript ts=2 et sw=2 tw=80: */
 /* Any copyright is dedicated to the Public Domain.
  http://creativecommons.org/publicdomain/zero/1.0/ */
 
@@ -10,7 +9,7 @@ const TEST_URI = "<h1 style='font: 24px serif'></h1>";
 
 add_task(async function() {
   await addTab("data:text/html;charset=utf-8," + encodeURIComponent(TEST_URI));
-  const {inspector, view, testActor} = await openRuleView();
+  const { inspector, view } = await openRuleView();
 
   info("Test autocompletion popup is hidden after page navigation");
 
@@ -18,8 +17,9 @@ add_task(async function() {
   await selectNode("h1", inspector);
 
   info("Focusing the css property editable field");
-  const propertyName = view.styleDocument
-    .querySelectorAll(".ruleview-propertyname")[0];
+  const propertyName = view.styleDocument.querySelectorAll(
+    ".ruleview-propertyname"
+  )[0];
   const editor = await focusEditableField(view, propertyName);
 
   info("Pressing key VK_DOWN");
@@ -35,7 +35,7 @@ add_task(async function() {
   ok(view.popup && view.popup.isOpen, "Popup should be opened");
 
   info("Reloading the page");
-  await reloadPage(inspector, testActor);
+  await reloadBrowser();
 
   ok(!(view.popup && view.popup.isOpen), "Popup should be closed");
 });
