@@ -27,6 +27,10 @@ const testCases = [
 ];
 
 add_task(async function() {
+  // storage-listings.html explicitly mixes secure and insecure frames.
+  // We should not enforce https for tests using this page.
+  await pushPref("dom.security.https_first", false);
+
   await openTabAndSetupStorage(MAIN_DOMAIN + "storage-listings.html");
 
   for (const test of testCases) {
@@ -43,15 +47,19 @@ add_task(async function() {
     if (clickToggle) {
       toggleSidebar();
     } else if (typeof toggleButtonHidden !== "undefined") {
-      is(sidebarToggleVisible(), toggleButtonVisible,
-         "correct visibility state of toggle button");
+      is(
+        sidebarToggleVisible(),
+        toggleButtonVisible,
+        "correct visibility state of toggle button"
+      );
     } else {
-      is(gUI.sidebar.hidden, sidebarHidden,
-        "correct visibility state of sidebar.");
+      is(
+        gUI.sidebar.hidden,
+        sidebarHidden,
+        "correct visibility state of sidebar."
+      );
     }
 
     info("-".repeat(80));
   }
-
-  await finishTests();
 });
