@@ -16,7 +16,9 @@ const {
   isNativeAnonymous,
 } = require("devtools/shared/layout/utils");
 const Debugger = require("Debugger");
-const ReplayInspector = require("devtools/server/actors/replay/inspector");
+const {
+  EXCLUDED_LISTENER,
+} = require("devtools/server/actors/inspector/constants");
 
 // eslint-disable-next-line
 const JQUERY_LIVE_REGEX = /return typeof \w+.*.event\.triggered[\s\S]*\.event\.(dispatch|handle).*arguments/;
@@ -202,7 +204,9 @@ class MainEventCollector {
    */
   get chromeEnabled() {
     if (typeof this._chromeEnabled === "undefined") {
-      this._chromeEnabled = Services.prefs.getBoolPref("devtools.chrome.enabled");
+      this._chromeEnabled = Services.prefs.getBoolPref(
+        "devtools.chrome.enabled"
+      );
     }
 
     return this._chromeEnabled;
@@ -238,7 +242,7 @@ class MainEventCollector {
    * @return {Array}
    *         An array of event handlers.
    */
-  getListeners(node, {checkOnly}) {
+  getListeners(node, { checkOnly }) {
     throw new Error("You have to implement the method getListeners()!");
   }
 
@@ -253,17 +257,26 @@ class MainEventCollector {
    *         An array of unfiltered event listeners or an empty array
    */
   getDOMListeners(node) {
-    if (typeof node.nodeName !== "undefined" && node.nodeName.toLowerCase() === "html") {
+    let listeners;
+    if (
+      typeof node.nodeName !== "undefined" &&
+      node.nodeName.toLowerCase() === "html"
+    ) {
       const winListeners =
         Services.els.getListenerInfoFor(node.ownerGlobal) || [];
-      const docElementListeners =
-        Services.els.getListenerInfoFor(node) || [];
+      const docElementListeners = Services.els.getListenerInfoFor(node) || [];
       const docListeners =
         Services.els.getListenerInfoFor(node.parentNode) || [];
 
-      return [...winListeners, ...docElementListeners, ...docListeners];
+      listeners = [...winListeners, ...docElementListeners, ...docListeners];
+    } else {
+      listeners = Services.els.getListenerInfoFor(node) || [];
     }
-    return Services.els.getListenerInfoFor(node) || [];
+
+    return listeners.filter(listener => {
+      const obj = this.unwrap(listener.listenerObject);
+      return !obj || !obj[EXCLUDED_LISTENER];
+    });
   }
 
   getJQuery(node) {
@@ -276,7 +289,7 @@ class MainEventCollector {
       return null;
     }
 
-    const hasJQuery = global.jQuery && global.jQuery.fn && global.jQuery.fn.jquery;
+    const hasJQuery = global.jQuery?.fn?.jquery;
 
     if (hasJQuery) {
       return global.jQuery;
@@ -289,13 +302,22 @@ class MainEventCollector {
   }
 
   isChromeHandler(handler) {
-    const handlerPrincipal = Cu.getObjectPrincipal(handler);
+    try {
+      const handlerPrincipal = Cu.getObjectPrincipal(handler);
 
-    // Chrome codebase may register listeners on the page from a frame script or
-    // JSM <video> tags may also report internal listeners, but they won't be
-    // coming from the system principal. Instead, they will be using an expanded
-    // principal.
-    return handlerPrincipal.isSystemPrincipal || handlerPrincipal.isExpandedPrincipal;
+      // Chrome codebase may register listeners on the page from a frame script or
+      // JSM <video> tags may also report internal listeners, but they won't be
+      // coming from the system principal. Instead, they will be using an expanded
+      // principal.
+      return (
+        handlerPrincipal.isSystemPrincipal ||
+        handlerPrincipal.isExpandedPrincipal
+      );
+    } catch (e) {
+      // Anything from a dead object to a CSP error can leave us here so let's
+      // return false so that we can fail gracefully.
+      return false;
+    }
   }
 }
 
@@ -307,7 +329,7 @@ class MainEventCollector {
  * through `processHandlerForEvent()`.
  */
 class DOMEventCollector extends MainEventCollector {
-  getListeners(node, {checkOnly} = {}) {
+  getListeners(node, { checkOnly } = {}) {
     const handlers = [];
     const listeners = this.getDOMListeners(node);
 
@@ -363,9 +385,11 @@ class DOMEventCollector extends MainEventCollector {
       }
 
       const eventInfo = {
+        nsIEventListenerInfo: listener,
         capturing: listener.capturing,
         type: listener.type,
         handler: handler,
+        enabled: listener.enabled,
       };
 
       handlers.push(eventInfo);
@@ -385,14 +409,20 @@ class DOMEventCollector extends MainEventCollector {
  * Get or detect jQuery events.
  */
 class JQueryEventCollector extends MainEventCollector {
-  getListeners(node, {checkOnly} = {}) {
+  // eslint-disable-next-line complexity
+  getListeners(node, { checkOnly } = {}) {
     const jQuery = this.getJQuery(node);
     const handlers = [];
 
     // If jQuery is not on the page, if this is an anonymous node or a pseudo
     // element we need to return early.
-    if (!jQuery || isNativeAnonymous(node) || isMarkerPseudoElement(node) ||
-        isBeforePseudoElement(node) || isAfterPseudoElement(node)) {
+    if (
+      !jQuery ||
+      isNativeAnonymous(node) ||
+      isMarkerPseudoElement(node) ||
+      isBeforePseudoElement(node) ||
+      isAfterPseudoElement(node)
+    ) {
       if (checkOnly) {
         return false;
       }
@@ -456,7 +486,6 @@ class JQueryEventCollector extends MainEventCollector {
               tags: "jQuery",
               hide: {
                 capturing: true,
-                dom0: true,
               },
             };
 
@@ -477,7 +506,8 @@ class JQueryEventCollector extends MainEventCollector {
  * Get or detect jQuery live events.
  */
 class JQueryLiveEventCollector extends MainEventCollector {
-  getListeners(node, {checkOnly} = {}) {
+  // eslint-disable-next-line complexity
+  getListeners(node, { checkOnly } = {}) {
     const jQuery = this.getJQuery(node);
     const handlers = [];
 
@@ -548,12 +578,11 @@ class JQueryLiveEventCollector extends MainEventCollector {
                 handler: handler,
                 tags: "jQuery,Live",
                 hide: {
-                  dom0: true,
                   capturing: true,
                 },
               };
 
-              if (!eventInfo.type && event.data && event.data.live) {
+              if (!eventInfo.type && event.data?.live) {
                 eventInfo.type = event.data.live;
               }
 
@@ -584,11 +613,11 @@ class JQueryLiveEventCollector extends MainEventCollector {
       // function gets name at compile time by SetFunctionName, its guessed
       // atom doesn't contain "proxy/".  In that case, check if the caller is
       // "proxy" function, as a fallback.
-      const calleeDO = funcDO.environment.callee;
-      if (!calleeDO) {
+      const calleeDS = funcDO.environment.calleeScript;
+      if (!calleeDS) {
         return false;
       }
-      const calleeName = calleeDO.displayName;
+      const calleeName = calleeDS.displayName;
       return calleeName == "proxy";
     }
 
@@ -634,14 +663,14 @@ class JQueryLiveEventCollector extends MainEventCollector {
  * Get or detect React events.
  */
 class ReactEventCollector extends MainEventCollector {
-  getListeners(node, {checkOnly} = {}) {
+  getListeners(node, { checkOnly } = {}) {
     const handlers = [];
     const props = this.getProps(node);
 
     if (props) {
       for (const [name, prop] of Object.entries(props)) {
         if (REACT_EVENT_NAMES.includes(name)) {
-          const listener = prop && prop.__reactBoundMethod || prop;
+          const listener = prop?.__reactBoundMethod || prop;
 
           if (typeof listener !== "function") {
             continue;
@@ -659,9 +688,6 @@ class ReactEventCollector extends MainEventCollector {
             type: name,
             handler: listener,
             tags: "React",
-            hide: {
-              dom0: true,
-            },
             override: {
               capturing: name.endsWith("Capture"),
             },
@@ -688,7 +714,7 @@ class ReactEventCollector extends MainEventCollector {
         if (value.memoizedProps) {
           return value.memoizedProps; // React 16
         }
-        return value && value._currentElement && value._currentElement.props; // React 15
+        return value?._currentElement?.props; // React 15
       }
     }
     return null;
@@ -701,8 +727,10 @@ class ReactEventCollector extends MainEventCollector {
       handlerDO = handlerDO.boundTargetFunction;
     }
 
-    const introScript = handlerDO.script.source.introductionScript;
     const script = handlerDO.script;
+    // Script might be undefined (eg for methods bound several times, see
+    // https://bugzilla.mozilla.org/show_bug.cgi?id=1589658)
+    const introScript = script?.source.introductionScript;
 
     // If this is a Babel transpiled function we have no access to the
     // source location so we need to hide the filename and debugger
@@ -722,8 +750,10 @@ class ReactEventCollector extends MainEventCollector {
       functionText += ") {\n";
 
       const scriptSource = script.source.text;
-      functionText +=
-        scriptSource.substr(script.sourceStart, script.sourceLength);
+      functionText += scriptSource.substr(
+        script.sourceStart,
+        script.sourceLength
+      );
 
       listener.override.handler = functionText;
     }
@@ -780,7 +810,9 @@ class EventCollector {
    */
   get chromeEnabled() {
     if (typeof this._chromeEnabled === "undefined") {
-      this._chromeEnabled = Services.prefs.getBoolPref("devtools.chrome.enabled");
+      this._chromeEnabled = Services.prefs.getBoolPref(
+        "devtools.chrome.enabled"
+      );
     }
 
     return this._chromeEnabled;
@@ -790,17 +822,20 @@ class EventCollector {
    *
    * @param  {DOMNode} node
    *         The node for which events are to be gathered.
-   * @return {Array}
+   * @return {Array<Object>}
    *         An array containing objects in the following format:
    *         {
-   *           type: type,        // e.g. "click"
-   *           handler: handler,  // The function called when event is triggered.
-   *           tags: "jQuery",    // Comma separated list of tags displayed
-   *                              // inside event bubble.
-   *           hide: {            // Flags for hiding certain properties.
-   *             capturing: true,
-   *             dom0: true,
+   *           {String} type: The event type, e.g. "click"
+   *           {Function} handler: The function called when event is triggered.
+   *           {Boolean} enabled: Whether the listener is enabled or not (event listeners can
+   *                     be disabled via the inspector)
+   *           {String} tags: Comma separated list of tags displayed inside event bubble (e.g. "JQuery")
+   *           {Object} hide: Flags for hiding certain properties.
+   *             {Boolean} capturing
    *           }
+   *           {Boolean} native
+   *           {String|undefined} sourceActor: The sourceActor id of the event listener
+   *           {nsIEventListenerInfo|undefined} nsIEventListenerInfo
    *         }
    */
   getEventListeners(node) {
@@ -826,10 +861,14 @@ class EventCollector {
       }
 
       for (const listener of listeners) {
-        if (collector.normalizeListener) {
-          listener.normalizeListener = collector.normalizeListener;
+        const eventObj = this.processHandlerForEvent(
+          listener,
+          dbg,
+          collector.normalizeListener
+        );
+        if (eventObj) {
+          listenerArray.push(eventObj);
         }
-        this.processHandlerForEvent(listenerArray, listener, dbg);
       }
     }
 
@@ -843,13 +882,13 @@ class EventCollector {
   /**
    * Process an event listener.
    *
-   * @param  {Array} listenerArray
-   *         listenerArray contains all event objects that we have gathered
-   *         so far.
    * @param  {EventListener} listener
    *         The event listener to process.
    * @param  {Debugger} dbg
    *         Debugger instance.
+   * @param  {Function|null} normalizeListener
+   *         An optional function that will be called to retrieve data about the listener.
+   *         It should be a *Collector method.
    *
    * @return {Array}
    *         An array of objects where a typical object looks like this:
@@ -858,34 +897,31 @@ class EventCollector {
    *             handler: function() { doSomething() },
    *             origin: "http://www.mozilla.com",
    *             tags: tags,
-   *             DOM0: true,
    *             capturing: true,
    *             hide: {
-   *               DOM0: true
+   *               capturing: true
    *             },
-   *             native: false
+   *             native: false,
+   *             enabled: true
+   *             sourceActor: "sourceActor.1234",
+   *             nsIEventListenerInfo: nsIEventListenerInfo {…},
    *           }
    */
-  processHandlerForEvent(listenerArray, listener, dbg) {
+  // eslint-disable-next-line complexity
+  processHandlerForEvent(listener, dbg, normalizeListener) {
     let globalDO;
+    let eventObj;
 
     try {
       const { capturing, handler } = listener;
 
-      let listenerDO;
-      if (isReplaying) {
-        listenerDO = ReplayInspector.getDebuggerObject(handler);
-      } else {
-        const global = Cu.getGlobalForObject(handler);
+      const global = Cu.getGlobalForObject(handler);
 
-        // It is important that we recreate the globalDO for each handler because
-        // their global object can vary e.g. resource:// URLs on a video control. If
-        // we don't do this then all chrome listeners simply display "native code."
-        globalDO = dbg.addDebuggee(global);
-        listenerDO = globalDO.makeDebuggeeValue(handler);
-      }
-
-      const { normalizeListener } = listener;
+      // It is important that we recreate the globalDO for each handler because
+      // their global object can vary e.g. resource:// URLs on a video control. If
+      // we don't do this then all chrome listeners simply display "native code."
+      globalDO = dbg.addDebuggee(global);
+      let listenerDO = globalDO.makeDebuggeeValue(handler);
 
       if (normalizeListener) {
         listenerDO = normalizeListener(listenerDO, listener);
@@ -895,15 +931,19 @@ class EventCollector {
       const override = listener.override || {};
       const tags = listener.tags || "";
       const type = listener.type || "";
-      let dom0 = false;
+      const enabled = !!listener.enabled;
       let functionSource = handler.toString();
       let line = 0;
+      let column = null;
       let native = false;
       let url = "";
       let sourceActor = "";
 
       // If the listener is an object with a 'handleEvent' method, use that.
-      if (listenerDO.class === "Object" || /^XUL\w*Element$/.test(listenerDO.class)) {
+      if (
+        listenerDO.class === "Object" ||
+        /^XUL\w*Element$/.test(listenerDO.class)
+      ) {
         let desc;
 
         while (!desc && listenerDO) {
@@ -911,7 +951,7 @@ class EventCollector {
           listenerDO = listenerDO.proto;
         }
 
-        if (desc && desc.value) {
+        if (desc?.value) {
           listenerDO = desc.value;
         }
       }
@@ -927,27 +967,26 @@ class EventCollector {
       if (script) {
         const scriptSource = script.source.text;
 
-        // Scripts are provided via script tags. If it wasn't provided by a
-        // script tag it must be a DOM0 event.
-        if (script.source.element) {
-          dom0 = script.source.element.class !== "HTMLScriptElement";
-        } else {
-          dom0 = false;
-        }
-
         line = script.startLine;
+        column = script.startColumn;
         url = script.url;
-        const actor = this.targetActor.sources.getOrCreateSourceActor(script.source);
+        const actor = this.targetActor.sourcesManager.getOrCreateSourceActor(
+          script.source
+        );
         sourceActor = actor ? actor.actorID : null;
 
         // Checking for the string "[native code]" is the only way at this point
         // to check for native code. Even if this provides a false positive then
         // grabbing the source code a second time is harmless.
-        if (functionSource === "[object Object]" ||
-            functionSource === "[object XULElement]" ||
-            functionSource.includes("[native code]")) {
-          functionSource =
-            scriptSource.substr(script.sourceStart, script.sourceLength);
+        if (
+          functionSource === "[object Object]" ||
+          functionSource === "[object XULElement]" ||
+          functionSource.includes("[native code]")
+        ) {
+          functionSource = scriptSource.substr(
+            script.sourceStart,
+            script.sourceLength
+          );
 
           // At this point the script looks like this:
           // () { ... }
@@ -983,35 +1022,41 @@ class EventCollector {
       if (native) {
         origin = "[native code]";
       } else {
-        origin = url + ((dom0 || line === 0) ? "" : ":" + line);
+        origin =
+          url +
+          (line ? ":" + line + (column === null ? "" : ":" + column) : "");
       }
 
-      const eventObj = {
+      eventObj = {
         type: override.type || type,
         handler: override.handler || functionSource.trim(),
         origin: override.origin || origin,
         tags: override.tags || tags,
-        DOM0: typeof override.dom0 !== "undefined" ? override.dom0 : dom0,
-        capturing: typeof override.capturing !== "undefined" ?
-                          override.capturing : capturing,
+        capturing:
+          typeof override.capturing !== "undefined"
+            ? override.capturing
+            : capturing,
         hide: typeof override.hide !== "undefined" ? override.hide : hide,
         native,
         sourceActor,
+        nsIEventListenerInfo: listener.nsIEventListenerInfo,
+        enabled,
       };
 
       // Hide the debugger icon for DOM0 and native listeners. DOM0 listeners are
       // generated dynamically from e.g. an onclick="" attribute so the script
       // doesn't actually exist.
-      if (native || dom0) {
+      if (!sourceActor) {
         eventObj.hide.debugger = true;
       }
-      listenerArray.push(eventObj);
     } finally {
       // Ensure that we always remove the debuggee.
       if (globalDO) {
         dbg.removeDebuggee(globalDO);
       }
     }
+
+    return eventObj;
   }
 }
 
