@@ -5,11 +5,13 @@
 #include "PreloaderBase.h"
 
 #include "mozilla/dom/Document.h"
+#include "mozilla/Telemetry.h"
+#include "nsContentUtils.h"
 #include "nsIAsyncVerifyRedirectCallback.h"
 #include "nsIChannel.h"
 #include "nsILoadGroup.h"
 #include "nsIInterfaceRequestorUtils.h"
-#include "nsProxyRelease.h"
+#include "nsIRedirectResultListener.h"
 
 // Change this if we want to cancel and remove the associated preload on removal
 // of all <link rel=preload> tags from the tree.
@@ -17,6 +19,10 @@ constexpr static bool kCancelAndRemovePreloadOnZeroReferences = false;
 
 namespace mozilla {
 
+PreloaderBase::UsageTimer::UsageTimer(PreloaderBase* aPreload,
+                                      dom::Document* aDocument)
+    : mDocument(aDocument), mPreload(aPreload) {}
+
 class PreloaderBase::RedirectSink final : public nsIInterfaceRequestor,
                                           public nsIChannelEventSink,
                                           public nsIRedirectResultListener {
@@ -32,7 +38,7 @@ class PreloaderBase::RedirectSink final : public nsIInterfaceRequestor,
   RedirectSink(PreloaderBase* aPreloader, nsIInterfaceRequestor* aCallbacks);
 
  private:
-  WeakPtr<PreloaderBase> mPreloader;
+  MainThreadWeakPtr<PreloaderBase> mPreloader;
   nsCOMPtr<nsIInterfaceRequestor> mCallbacks;
   nsCOMPtr<nsIChannel> mRedirectChannel;
 };
@@ -41,10 +47,7 @@ PreloaderBase::RedirectSink::RedirectSink(PreloaderBase* aPreloader,
                                           nsIInterfaceRequestor* aCallbacks)
     : mPreloader(aPreloader), mCallbacks(aCallbacks) {}
 
-PreloaderBase::RedirectSink::~RedirectSink() {
-  NS_ReleaseOnMainThread("RedirectSink::mPreloader::mRef",
-                         mPreloader.TakeRef());
-}
+PreloaderBase::RedirectSink::~RedirectSink() = default;
 
 NS_IMPL_ISUPPORTS(PreloaderBase::RedirectSink, nsIInterfaceRequestor,
                   nsIChannelEventSink, nsIRedirectResultListener)
@@ -119,28 +122,34 @@ void PreloaderBase::AddLoadBackgroundFlag(nsIChannel* aChannel) {
   aChannel->SetLoadFlags(loadFlags | nsIRequest::LOAD_BACKGROUND);
 }
 
-void PreloaderBase::NotifyOpen(PreloadHashKey* aKey, dom::Document* aDocument,
-                               bool aIsPreload) {
-  if (aDocument && !aDocument->Preloads().RegisterPreload(aKey, this)) {
+void PreloaderBase::NotifyOpen(const PreloadHashKey& aKey,
+                               dom::Document* aDocument, bool aIsPreload) {
+  if (aDocument) {
+    DebugOnly<bool> alreadyRegistered =
+        aDocument->Preloads().RegisterPreload(aKey, this);
     // This means there is already a preload registered under this key in this
     // document.  We only allow replacement when this is a regular load.
     // Otherwise, this should never happen and is a suspected misuse of the API.
-    MOZ_ASSERT(!aIsPreload);
-    aDocument->Preloads().DeregisterPreload(aKey);
-    aDocument->Preloads().RegisterPreload(aKey, this);
+    MOZ_ASSERT_IF(alreadyRegistered, !aIsPreload);
   }
 
-  mKey = *aKey;
+  mKey = aKey;
   mIsUsed = !aIsPreload;
+
+  if (!mIsUsed && !mUsageTimer) {
+    auto callback = MakeRefPtr<UsageTimer>(this, aDocument);
+    NS_NewTimerWithCallback(getter_AddRefs(mUsageTimer), callback, 10000,
+                            nsITimer::TYPE_ONE_SHOT);
+  }
+
+  ReportUsageTelemetry();
 }
 
-void PreloaderBase::NotifyOpen(PreloadHashKey* aKey, nsIChannel* aChannel,
+void PreloaderBase::NotifyOpen(const PreloadHashKey& aKey, nsIChannel* aChannel,
                                dom::Document* aDocument, bool aIsPreload) {
   NotifyOpen(aKey, aDocument, aIsPreload);
   mChannel = aChannel;
 
-  // * Start the usage timer if aIsPreload.
-
   nsCOMPtr<nsIInterfaceRequestor> callbacks;
   mChannel->GetNotificationCallbacks(getter_AddRefs(callbacks));
   RefPtr<RedirectSink> sink(new RedirectSink(this, callbacks));
@@ -174,13 +183,13 @@ void PreloaderBase::NotifyUsage(LoadBackground aLoadBackground) {
   }
 
   mIsUsed = true;
-
-  // * Cancel the usage timer.
+  ReportUsageTelemetry();
+  CancelUsageTimer();
 }
 
 void PreloaderBase::RemoveSelf(dom::Document* aDocument) {
   if (aDocument) {
-    aDocument->Preloads().DeregisterPreload(&mKey);
+    aDocument->Preloads().DeregisterPreload(mKey);
   }
 }
 
@@ -189,6 +198,8 @@ void PreloaderBase::NotifyRestart(dom::Document* aDocument,
   RemoveSelf(aDocument);
   mKey = PreloadHashKey();
 
+  CancelUsageTimer();
+
   if (aNewPreloader) {
     aNewPreloader->mNodes = std::move(mNodes);
   }
@@ -230,8 +241,7 @@ void PreloaderBase::NotifyStop(nsIRequest* aRequest, nsresult aStatus) {
 void PreloaderBase::NotifyStop(nsresult aStatus) {
   mOnStopStatus.emplace(aStatus);
 
-  nsTArray<nsWeakPtr> nodes;
-  nodes.SwapElements(mNodes);
+  nsTArray<nsWeakPtr> nodes = std::move(mNodes);
 
   for (nsWeakPtr& weak : nodes) {
     nsCOMPtr<nsINode> node = do_QueryReferent(weak);
@@ -282,6 +292,34 @@ void PreloaderBase::NotifyNodeEvent(nsINode* aNode) {
       aNode, mShouldFireLoadEvent || NS_SUCCEEDED(*mOnStopStatus));
 }
 
+void PreloaderBase::CancelUsageTimer() {
+  if (mUsageTimer) {
+    mUsageTimer->Cancel();
+    mUsageTimer = nullptr;
+  }
+}
+
+void PreloaderBase::ReportUsageTelemetry() {
+  if (mUsageTelementryReported) {
+    return;
+  }
+  mUsageTelementryReported = true;
+
+  if (mKey.As() == PreloadHashKey::ResourceType::NONE) {
+    return;
+  }
+
+  // The labels are structured as type1-used, type1-unused, type2-used, ...
+  // The first "as" resource type is NONE with value 0.
+  auto index = (static_cast<uint32_t>(mKey.As()) - 1) * 2;
+  if (!mIsUsed) {
+    ++index;
+  }
+
+  auto label = static_cast<Telemetry::LABELS_REL_PRELOAD_MISS_RATIO>(index);
+  Telemetry::AccumulateCategorical(label);
+}
+
 nsresult PreloaderBase::AsyncConsume(nsIStreamListener* aListener) {
   // We want to return an error so that consumers can't ever use a preload to
   // consume data unless it's properly implemented.
@@ -303,4 +341,46 @@ nsCString PreloaderBase::RedirectRecord::Fragment() const {
   return fragment;
 }
 
+// PreloaderBase::UsageTimer
+
+NS_IMPL_ISUPPORTS(PreloaderBase::UsageTimer, nsITimerCallback, nsINamed)
+
+NS_IMETHODIMP PreloaderBase::UsageTimer::Notify(nsITimer* aTimer) {
+  if (!mPreload || !mDocument) {
+    return NS_OK;
+  }
+
+  MOZ_ASSERT(aTimer == mPreload->mUsageTimer);
+  mPreload->mUsageTimer = nullptr;
+
+  if (mPreload->IsUsed()) {
+    // Left in the hashtable, but marked as used.  This is a valid case, and we
+    // don't want to emit a warning for this preload then.
+    return NS_OK;
+  }
+
+  mPreload->ReportUsageTelemetry();
+
+  // PreloadHashKey overrides GetKey, we need to use the nsURIHashKey one to get
+  // the URI.
+  nsIURI* uri = static_cast<nsURIHashKey*>(&mPreload->mKey)->GetKey();
+  if (!uri) {
+    return NS_OK;
+  }
+
+  nsString spec;
+  NS_GetSanitizedURIStringFromURI(uri, spec);
+  nsContentUtils::ReportToConsole(nsIScriptError::warningFlag, "DOM"_ns,
+                                  mDocument, nsContentUtils::eDOM_PROPERTIES,
+                                  "UnusedLinkPreloadPending",
+                                  nsTArray<nsString>({std::move(spec)}));
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+PreloaderBase::UsageTimer::GetName(nsACString& aName) {
+  aName.AssignLiteral("PreloaderBase::UsageTimer");
+  return NS_OK;
+}
+
 }  // namespace mozilla
